// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file PetscSolverSchur.h */

#include "parallel/PetscSolver.h"

#ifndef AMDIS_PETSC_SOLVER_SCHUR_H
#define AMDIS_PETSC_SOLVER_SCHUR_H

namespace AMDiS {

  using namespace std;


  class PetscSolverSchur : public PetscSolver
  {
  public:
    PetscSolverSchur() 
      : PetscSolver()
    {}

    void fillPetscMatrix(Matrix<DOFMatrix*> *mat);

    void fillPetscRhs(SystemVector *vec);

    void solvePetscMatrix(SystemVector &vec, AdaptInfo *adaptInfo);

    void destroyMatrixData() 
    {}

    void destroyVectorData() 
    {}

    Flag getBoundaryDofRequirement()
    {
      return 
	MeshDistributor::BOUNDARY_SUBOBJ_SORTED |
	MeshDistributor::BOUNDARY_FILL_INFO_SEND_DOFS;
    }
    
  protected:
    void updateDofData(int nComponents);

    /// Takes a DOF matrix and sends the values to the global PETSc matrix.
    void setDofMatrix(DOFMatrix* mat, int dispMult = 1, 
		      int dispAddRow = 0, int dispAddCol = 0);

    /// Takes a DOF vector and sends its values to a given PETSc vector.
    void setDofVector(Vec& petscVec, DOFVector<double>* vec, 
		      int disMult = 1, int dispAdd = 0, bool rankOnly = false);

  protected:
    int nBoundaryDofs;

    int rStartBoundaryDofs;

    int nOverallBoundaryDofs;

    std::set<DegreeOfFreedom> boundaryDofs;

    map<DegreeOfFreedom, DegreeOfFreedom> mapGlobalBoundaryDof;

    int nInteriorDofs;

    int rStartInteriorDofs;

    int nOverallInteriorDofs;

    std::set<DegreeOfFreedom> interiorDofs;    

    map<DegreeOfFreedom, DegreeOfFreedom> mapGlobalInteriorDof;

    Mat matA11, matA12, matA21, matA22;

    IS interiorIs, boundaryIs;
  };

}

#endif
